#include <iostream>
#include <vector>
using namespace std;


class Parser;


//stack 
template <class T> class Stack : public vector<T> {
public:
	//constructor
	Stack() : m_top(-1) {
	}

	//empty
	bool empty() const {
		return m_top == -1;
	}

	//push element
	void push(const T &elem) {
		++m_top;
		if (m_top == size()) resize(size() ? size() * 2 : 16);
		(*this)[m_top] = elem;
	}

	//pop
	void pop() {
		--m_top;
	}

	//get top
	int top() const {
		return m_top;
	}

	//set top 
	void setTop(int top) {
		m_top = top;
	}

	//get top element
	const T &topElem() const {
		return (*this)[m_top];
	}

private:
	//top element
	int m_top;
};


class Context;


//base class for parsers
class Parser {
public:
	//destructor
	virtual ~Parser() {
	}

	//parse interface
	virtual bool parse(Context &context) const = 0;
};


//parser context
class Context {
public:
	//constructor
	Context() : m_tokenPosition(0), m_tokenRead(-1) {
	}

	//push continuation
	void pushContinuation(Parser *parser) {
		m_continuations.push(parser);
	}

	//pops a continuation
	Parser *popContinuation() {
		Parser *result = m_continuations.topElem();
		m_continuations.pop();
		return result;
	}

	//push backtracking
	void pushBacktracking(Parser *parser = 0) {
		pushContinuation(&m_cancelBacktracking);
		m_backtrackings.push(Backtracking(m_tokenPosition, m_continuations.top() - 1, parser));
	}

	//activate backtracking
	bool activateBacktracking() {
		if (m_backtrackings.empty()) return false;
		const Backtracking &bt = m_backtrackings.topElem();
		m_tokenPosition = bt.m_tokenPosition;
		m_continuations.setTop(bt.m_contPosition);
		if (bt.m_parser) m_continuations.push(bt.m_parser);
		m_backtrackings.pop();
		return true;
	}

	//cancel backtracking
	void cancelBacktracking() {
		m_backtrackings.pop();
	}

	//matches a token
	bool matchToken(char token) {
		char input = m_tokens[m_tokenPosition];
		if (input != token) return false;
		++m_tokenPosition;
		return true;
	}

	//get input from stream if required; returns true if eof
	template <class T> bool getInput(T &stream) {
		//if need more tokens
		if (m_tokenPosition > m_tokenRead) {
			++m_tokenRead;

			//if required, make room for new tokens
			if (m_tokenRead == m_tokens.size()) {
				m_tokens.resize(m_tokens.size() ? m_tokens.size() * 2 : 16);
			}

			//read token
			char token;
			stream >> token;

			//if a token was available, add token
			if (!stream.eof()) m_tokens[m_tokenRead] = token;
		}

		//continue parsing
		return stream.eof();
	}

private:
	//backtracking
	class Backtracking {
	public:
		//constructor
		Backtracking(int tokenPos = 0, int contPos = 0, Parser *parser = 0) : 
			m_tokenPosition(tokenPos), m_contPosition(contPos), m_parser(parser) {
		}

	private:
		//index in tokens
		int m_tokenPosition;

		//index in continuations
		int m_contPosition;

		//parser
		Parser *m_parser;

		friend class Context;
	};

	//special parser for cancelling backtracking, once the 1st branch is successful
	class CancelBacktracking : public Parser {
	public:
		//parse
		virtual bool parse(Context &context) const {
			context.cancelBacktracking();
			return true;
		}
	};

	//tokens read so far
	vector<char> m_tokens;

	//current token; index into tokens
	int m_tokenPosition;

	//index of top token read
	int m_tokenRead;

	//continuations
	Stack<Parser *> m_continuations;

	//backtrackings
	Stack<Backtracking> m_backtrackings;

	//special parser to cancel backtracking
	CancelBacktracking m_cancelBacktracking;

	friend class State;
};


//special parser used for end
class End : public Parser {
public:
	//parse
	virtual bool parse(Context &context) const {
		return true;
	}
};


//atom parser
class Atom : public Parser {
public:
	//constructor
	Atom(char value) : m_value(value) {
	}

	//parse
	virtual bool parse(Context &context) const {
		return context.matchToken(m_value);
	}

private:
	//value
	char m_value;		
};


//sequence
class Sequence : public Parser {
public:
	//constructor
	Sequence(Parser *left, Parser *right) : m_left(left), m_right(right) {
	}

	//parse
	virtual bool parse(Context &context) const {
		context.pushContinuation(m_right);
		context.pushContinuation(m_left);
		return true;
	}

private:
	//operands
	Parser *m_left;
	Parser *m_right;
};


//sequence
class Choice : public Parser {
public:
	//constructor
	Choice(Parser *left, Parser *right) : m_left(left), m_right(right) {
	}

	//parse
	virtual bool parse(Context &context) const {
		context.pushBacktracking(m_right);
		context.pushContinuation(m_left);
		return true;
	}

private:
	//operands
	Parser *m_left;
	Parser *m_right;
};


//repeat 0 or more times
class Repetition : public Parser {
public:
	//constructor
	Repetition(Parser *parser) : m_parser(parser) {
	}

	//parse
	virtual bool parse(Context &context) const {
		context.pushBacktracking();
		context.pushContinuation(const_cast<Repetition *>(this));
		context.pushContinuation(m_parser);
		return true;
	}

private:
	//operand
	Parser *m_parser;
};


//optional
class Optional : public Parser {
public:
	//constructor
	Optional(Parser *parser) : m_parser(parser) {
	}

	//parse
	virtual bool parse(Context &context) const {
		context.pushBacktracking();
		context.pushContinuation(m_parser);
		return true;
	}

private:
	//operand
	Parser *m_parser;
};


class Rule;


//rule ref; used for recursion
class RuleRef : public Parser {
public:
	//constructor
	RuleRef(const Rule &rule) : m_rule(rule) {
	}

	//parse
	virtual bool parse(Context &context) const;

private:
	//rule ref
	const Rule &m_rule;
};


//grammar rule
class Rule {
public:
	//generic constructor
	Rule(Parser *parser = 0) : m_parser(parser) {
	}

	//atom constructor
	Rule(char atom) : m_parser(new Atom(atom)) {
	}

	//string constructor
	Rule(const char *str) : m_parser(createStringParser(str)) {
	}

	//used as parser ptr
	operator Parser *() const {
		return m_parser;
	}

	//get members
	Parser *operator ->() const {
		return m_parser;
	}

	//create a sequence
	friend Rule operator & (const Rule &a, const Rule &b) {
		return new Sequence(a, b);
	}

	//create a rule
	friend Rule operator | (const Rule &a, const Rule &b) {
		return new Choice(a, b);
	}

	//repeat 0 or more times
	Rule operator *() const {
		return new Repetition(m_parser);	
	}

	//repeat 1 or more times
	Rule operator +() const {
		return new Sequence(m_parser, new Repetition(m_parser));	
	}

	//optional
	Rule operator !() const {
		return new Optional(m_parser);	
	}

	//recursive
	Rule operator &() const {
		return new RuleRef(*this);	
	}

private:
	//operand
	Parser *m_parser;

	//create string parser
	static Parser *createStringParser(const char *str) {
		if (*(str + 1)) return new Sequence(new Atom(*str), createStringParser(str + 1));
		return new Atom(*str);
	}
};


//parse
bool RuleRef::parse(Context &context) const {
	return m_rule->parse(context);
}


//create atom
Rule atom(char c) {
	return new Atom(c);
}


//create a choice between rules
Rule oneOf(
	const Rule &r1, 
	const Rule &r2, 
	const Rule &r3, 
	const Rule &r4, 
	const Rule &r5, 
	const Rule &r6, 
	const Rule &r7, 
	const Rule &r8, 
	const Rule &r9, 
	const Rule &r10) {
	return new Choice(r1, new Choice(r2, new Choice(r3, new Choice(r4, new Choice(r5, new Choice(r7, new Choice(r8, new Choice(r9, r10))))))));
}


//parse loop
template <class T> bool parse(const Rule &rootRule, T &inputStream) {
	//the parser context
	Context context;

	//install the termination parser
	End end;
	context.pushContinuation(&end);

	//install the first parser
	context.pushContinuation(rootRule);

	//loop until no more parsers or no more input is available
	while (1) {
		//get input, if required
		bool inputExhausted = context.getInput(inputStream);

		//get next parser
		Parser *parser = context.popContinuation();

		//if end is reached, return true if input exhausted
		if (parser == &end) return inputExhausted;

		//on successful parsing, continue loop
		if (parser->parse(context)) continue;

		//on error, backtrack; if backtracking is not available, end with an error
		if (!context.activateBacktracking()) return false;
	}

	//unreachable
	return false;
}


/*****************************************************************************/


#include <sstream>
using namespace std;


//forward references
extern Rule expr;


//grammar
Rule whitespace = *atom(' ');
Rule group = '(' & &expr & ')';
Rule digit = oneOf('0', '1', '2', '3', '4', '5', '6', '7', '8', '9');
Rule integer = +digit;
Rule factor = whitespace & (integer | group);
Rule term = factor & *((whitespace & '*' & factor) | (whitespace & '/' & factor));
Rule expr = term & *((whitespace & '+' & term) | (whitespace & '-' & term));


//input
const char *input = "1 * 2 + ( 1 + 3 )";


//main
int main() {
	stringstream str;
	str << input;
	bool ok = parse(expr, str);
	cout << (ok ? "success" : "failure") << endl;
	getchar();
	return 0;
}
